#!/usr/bin/env python
"""
box.py, module definition of Box class.
"""
import re
from yattag import Doc
from markdown_utils import markdown2html


class Box(object):
  """
  Object for handling a box. It is an environment that can contains anythings (figures, tables, notes...).

  The syntax is:

  $box
  $style[style_options]
  $caption(caption_type)[caption_options]{caption}
  $content(content_type)[content_options]{content}
  $endbox

  Note:
  1. "$style[...]" is optional; it defines the style's options for the whole box; "style_options" are valid
     css syntax statements  (e.g. "font-variant:small-caps;");
  2. "$caption(...)[...]{...}" is optional: it defines the box's caption; the "caption_type" defines the
     caption prefixing "class" (e.g. "Fig." for figures): any sentences are valid; the "caption_options" defines
     the style's options of only caption: they are valid css syntax statements (e.g.  "padding:0 2%;"); the
     "caption" (inside the {} parenthesis) defines the caption text; note that "caption_type" and "caption_options"
     are optional thus the following statements are valid:
     + $caption[font-variant:small-caps;]{My caption without caption_type};
     + $caption{My caption without caption_type and caption_options};
  3. "$content(...)[...]{...}" is not optional: it defines the box's content; the "content_type" defines the type
     of the content that can be 'figure', 'table', 'note' and 'box' for generic environments; the "content_options"
     defines the style's options of only content: they are valid css syntax statements (e.g.  "padding:0 2%;"); the
     "content" (inside the {} parenthesis) defines the content (being text, figures, tables, etc...); note that
     "content_type" and "content_options" are optional thus the following statements are valid:
     + $content[font-variant:small-caps;]{My content without content_type};
     + $content{My content without content_type and content_options};

  There are some helper (sub)classes based on Box class for handling specific environments such Video, Figure, Table and Note.

  Note that the themes of box environments can be defined as all other theme elements in order to not have to repeat
  the styling options for each box. To this aim this module provides the "get_themes" function. The definition of such
  a theme can be stripped out by the function "strip_themes" also provided by this module.

  See Also
  --------
  Note
  Table
  Figure
  Video

  Attributes
  ----------
  regexs: dict
    dictionary of regexs
  boxes_number : int
    global number of boxes (equals to the number of Box instances)
  """
  regexs = {'box': re.compile(r"(?P<box>\$box(?P<env>.*?)\$endbox)", re.DOTALL),
            'style': re.compile(r"\$style\[(?P<style>.*?)\]", re.DOTALL),
            'content_fig': re.compile(r"\$content(\((?P<ctn_type>.*?)\))*(\[(?P<ctn_options>.*?)\])*\{(?P<ctn>.*?)\}", re.DOTALL),
            'content': re.compile(r"\$content(\((?P<ctn_type>.*?)\))*(\[(?P<ctn_options>.*?)\])*\{(?P<ctn>.*)\}", re.DOTALL),
            'caption': re.compile(r"\$caption(\((?P<cap_type>.*?)\))*(\[(?P<cap_options>.*?)\])*(\{(?P<cap>.*?)\})*", re.DOTALL),
            'caption_pos': re.compile(r"(?P<cap_pos>position\s*:\s*(?P<cap_position>(TOP)|(BOTTOM))\;*)", re.DOTALL)}
  boxes_number = 0

  @classmethod
  def reset(cls):
    """Reset to default state."""
    cls.boxes_number = 0
    return

  def __init__(self, ctn_type='box', source=None):
    """
    Parameters
    ----------
    ctn_type : {'video', 'figure', 'table', 'note', 'box'}, optional
      box content type
    source : str, optional
      string (as single stream) containing the source

    Attributes
    ----------
    number : int
      box number
    style : str
      box style
    ctn_type : {'video', 'figure', 'table', 'note', 'box'}
      box content type
    ctn_options : str
      box content options
    ctn : str
      box content
    ctn_type : {'video', 'figure', 'table', 'note'}
      box caption type
    cap_options : str
      box caption options
    cap_postion : {'TOP', 'BOTTOM'}
      box caption position
    cap : str
      box caption
    """
    self.number = 0
    self.ctn_type = ctn_type
    self.style = None
    self.ctn_options = None
    self.cap_options = None
    self.cap_position = None
    self.cap_type = None
    self.ctn = None
    self.cap = None
    if self.ctn_type == 'box':
      Box.boxes_number += 1
      self.number = Box.boxes_number
    if source is not None:
      self.get(source=source)
    return

  def __str__(self):
    string = []
    string.append('\nStyle: ' + str(self.style))
    string.append('\nCaption(' + str(self.cap_type) + ')[options=' + str(self.cap_options) + ']: ' + str(self.cap))
    string.append('\nContent(' + str(self.ctn_type) + ')[options=' + str(self.ctn_options) + ']: ' + str(self.ctn))
    return ''.join(string)

  def get_style(self, source):
    """Get box style data.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    """
    match = re.search(Box.regexs['style'], source)
    if match:
      style = match.group('style')
      if style:
        self.style = style.strip()
    return

  def get_caption(self, source):
    """Get box caption data.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    """
    match = re.search(Box.regexs['caption'], source)
    if match:
      cap_type = match.group('cap_type')
      if cap_type:
        if cap_type.lower() == 'none':
          self.cap_type = None
        else:
          self.cap_type = cap_type.strip()
      cap_options = match.group('cap_options')
      if cap_options:
        self.cap_options = cap_options.strip()
        cap_position = re.search(Box.regexs['caption_pos'], self.cap_options)
        if cap_position:
          self.cap_position = cap_position.group('cap_position').strip()
          self.cap_options = self.cap_options.replace(cap_position.group('cap_pos'), '')
          if self.cap_position != 'TOP' and self.cap_position != 'BOTTOM':
            self.cap_position = None
      self.cap = match.group('cap')
      if self.cap:
        self.cap = self.cap.strip()
    return

  def get_content(self, source):
    """Get box caption data.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    """
    if self.ctn_type == 'figure' or self.ctn_type == 'video':
      match = re.search(Box.regexs['content_fig'], source)
    else:
      match = re.search(Box.regexs['content'], source)
    if match:
      ctn_type = match.group('ctn_type')
      if ctn_type:
        self.ctn_type = ctn_type.strip()
      ctn_options = match.group('ctn_options')
      if ctn_options:
        self.ctn_options = ctn_options.strip()
      self.ctn = match.group('ctn')
      if self.ctn:
        self.ctn = self.ctn.strip()
        self.ctn = markdown2html(source=self.ctn, no_p=True)
    return

  def get(self, source):
    """
    Get box data from source.

    Parameters
    ----------
    source : str
      string containing source
    """
    self.get_style(source)
    self.get_caption(source)
    self.get_content(source)
    return

  def caption_txt(self):
    """Buildi caption text.

    Returns
    -------
    str
      caption text
    """
    if self.cap_type and self.cap:
      txt = self.cap_type + ' ' + str(self.number) + ': ' + markdown2html(source=self.cap, no_p=True)
    elif self.cap_type:
      txt = self.cap_type
    elif self.cap:
      txt = markdown2html(source=self.cap, no_p=True)
    return txt

  def put_caption(self, doc, klass='box caption'):
    """Method for inserting caption into doc.

    Parameters
    ----------
    doc : yattag.Doc object
      yattag document where to put caption
    """
    if self.cap or self.cap_type:
      with doc.tag('div', klass=klass):
        if self.cap_options:
          doc.attr(style=self.cap_options)
        doc.asis(self.caption_txt())
    return

  def to_html(self):
    """Convert self data to its html stream."""
    doc = Doc()
    with doc.tag('div', id='box-' + str(self.number)):
      if self.style:
        doc.attr(style=self.style)
      else:
        doc.attr(klass='box')
      if self.cap_position is not None and self.cap_position.upper() == 'TOP':
        self.put_caption(doc=doc, klass='box-caption')
      with doc.tag('div', klass='box-content'):
        if self.ctn_options:
          doc.attr(style=self.ctn_options)
        doc.asis(markdown2html(self.ctn, no_p=True))
      if self.cap_position is None or self.cap_position.upper() == 'BOTTOM':
        self.put_caption(doc=doc, klass='box-caption')
    return doc.getvalue()



#!/usr/bin/env python
"""
subsection.py, module definition of Subsection class.
"""
from __future__ import print_function
from slide import Slide


class Subsection(object):
  """
  Subsection object.

  Attributes
  ----------
  slides_number: int
  """
  slides_number = 0

  @classmethod
  def reset(cls):
    """Reset to default state."""
    cls.slides_number = 0
    Slide.reset()
    return

  def __init__(self, number, title=None):
    self.number = number
    self.title = title
    self.slides = []
    self.toc = []
    return

  def __str__(self):
    strings = [str(self.title)]
    for slide in self.slides:
      strings.append("      " + str(slide))
    return '\n'.join(strings)

  def update_toc(self):
    """Update TOC after a new slide (the last one) has been added."""
    self.toc.append(self.slides[-1].title)

  def add_slide(self, slide):
    """
    Add a slide to the subsection.

    Parameters
    ----------
    slide: Slide
    """
    Subsection.slides_number += 1
    self.slides.append(slide)
    self.update_toc()
    return

  def put_html_attributes(self, doc):
    """Put html attibutes of the subsection.

    Parameters
    ----------
    doc: Doc
    """
    doc.attr(('subsectionnumber', str(self.number)))
    if self.title is not None:
      doc.attr(('subsectiontitle', str(self.title)))
    return



#!/usr/bin/env python
"""
chapter.py, module definition of Chapter class.
"""
from __future__ import print_function
from collections import OrderedDict
from section import Section


class Chapter(object):
  """
  Chapter object.

  Attributes
  ----------
  sections_number: int
  """
  sections_number = 0

  @classmethod
  def reset(cls):
    """Reset to default state."""
    cls.sections_number = 0
    Section.reset()
    return

  def __init__(self, number, title=None):
    self.number = number
    self.title = title
    self.sections = []
    self.toc = OrderedDict()
    return

  def __str__(self):
    strings = [str(self.title)]
    for section in self.sections:
      strings.append("  " + str(section))
    return '\n'.join(strings)

  def update_toc(self):
    """Update TOC after a new section (the last one) has been added."""
    self.toc[self.sections[-1].title] = self.sections[-1].toc

  def add_section(self, section):
    """
    Add a section to the chapter.

    Parameters
    ----------
    section: Section
    """
    Chapter.sections_number += 1
    self.sections.append(section)
    self.update_toc()
    return

  def put_html_attributes(self, doc):
    """Put html attibutes of the chapter.

    Parameters
    ----------
    doc: Doc
    """
    doc.attr(('chapternumber', str(self.number)))
    if self.title is not None:
      doc.attr(('chaptertitle', str(self.title)))
    return





import markdown

class MathJaxPattern(markdown.inlinepatterns.Pattern):

    def __init__(self):
        markdown.inlinepatterns.Pattern.__init__(self, r'(?<!\\)(\$\$?)(.+?)\2')

    def handleMatch(self, m):
        node = markdown.util.etree.Element('mathjax')
        node.text = markdown.util.AtomicString(m.group(2) + m.group(3) + m.group(2))
        return node

class MathJaxExtension(markdown.Extension):
    def extendMarkdown(self, md, md_globals):
        # Needs to come before escape matching because \ is pretty important in LaTeX
        md.inlinePatterns.add('mathjax', MathJaxPattern(), '<escape')

def makeExtension(configs=[]):
    return MathJaxExtension(configs)





#!/usr/bin/env python
"""
MaTiSSe.py, Markdown To Impressive Scientific Slides
"""
from __future__ import print_function
import argparse
import os
import sys
from matisse_config import MatisseConfig
from presentation import Presentation

__appname__ = "MaTiSSe.py"
__description__ = "MaTiSSe.py, Markdown To Impressive Scientific Slides"
__long_description__ = "MaTiSSe.py, Markdown To Impressive Scientific Slides. It is a very simple and stupid-to-use (KISS) presentation maker based on simple markdown syntax producing high quality first-class html/css presentation with great support for scientific contents."
__version__ = "1.1.1"
__author__ = "Stefano Zaghi"
__author_email__ = "stefano.zaghi@gmail.com"
__license__ = "GNU General Public License v3 (GPLv3)"
__url__ = "https://github.com/szaghi/MaTiSSe"
__sample__ = r"""
---
theme:
  - backround: black
---

# Part 1

## Section 1

### Subsection 1

#### Slide 1

##### A H5 heading

Lorem ipsum dolor sit amet...

##### Math

$$
x=\frac{-b\pm\sqrt{b^2-4ac}}{2a}
$$

$note
$content{Just a note enviroment}
$endnote
"""


def main():
  """Main function."""
  cliparser = argparse.ArgumentParser(prog=__appname__, description='MaTiSSe.py, Markdown To Impressive Scientific Slides')
  cliparser.add_argument('-v', '--version', action='version', help='Show version', version='%(prog)s ' + __version__)
  cliparser.add_argument('-i', '--input', required=False, action='store', default=None, help='Input file name of markdown source to be parsed')
  cliparser.add_argument('-o', '--output', required=False, action='store', default=None, help='Output directory name containing the presentation files')
  cliparser.add_argument('-t', '--theme', required=False, action='store', default=None, help='Select a builtin theme for initializing a new sample presentation')
  cliparser.add_argument('-hs', '--highlight-style', required=False, action='store', default='github.css', help='Select the highlight.js style (default github.css); select "disable" to disable highligth.js', metavar='STYLE.CSS')
  cliparser.add_argument('-s', '--sample', required=False, action='store', default=None, help='Generate a new sample presentation as skeleton of your one')
  cliparser.add_argument('--toc-at-chap-beginning', required=False, action='store', default=None, help='Insert Table of Contents at each chapter beginning (default no): to activate indicate the TOC depth', metavar='TOC-DEPTH')
  cliparser.add_argument('--toc-at-sec-beginning', required=False, action='store', default=None, help='Insert Table of Contents at each section beginning (default no): to activate indicate the TOC depth', metavar='TOC-DEPTH')
  cliparser.add_argument('--toc-at-subsec-beginning', required=False, action='store', default=None, help='Insert Table of Contents at each subsection beginning (default no): to activate indicate the TOC depth', metavar='TOC-DEPTH')
  cliparser.add_argument('--print-preamble', required=False, action='store_true', default=None, help='Print the preamble data as parsed from source')
  cliparser.add_argument('--print-css', required=False, action='store_true', default=None, help='Print the css as parsed from source (if done)')
  cliparser.add_argument('--print-options', required=False, action='store_true', default=None, help='Print the available options for each presentation element')
  cliparser.add_argument('--print-highlight-styles', required=False, action='store_true', default=None, help='Print the available highlight.js style (default github.css)')
  cliparser.add_argument('--print-themes', required=False, action='store_true', default=None, help='Print the list of the builtin themes')
  cliparser.add_argument('--verbose', required=False, action='store_true', default=False, help='More verbose printing messages (default no)')
  cliparser.add_argument('--online-MathJax', required=False, action='store_true', default=None, help='Use online rendering of LaTeX equations by means of online MathJax service; default use offline, local copy of MathJax engine')
  cliargs = cliparser.parse_args()
  config = MatisseConfig(cliargs=cliargs)
  if cliargs.input:
    if not os.path.exists(cliargs.input):
      sys.stderr.write('Error: input file "' + cliargs.input + '" not found!')
      sys.exit(1)
    else:
      with open(cliargs.input, 'r') as mdf:
        source = mdf.read()
      presentation = Presentation()
      if config.verbose:
        print('Parsing source ' + cliargs.input)
      presentation.parse(config=config, source=source)
      if cliargs.output:
        output = cliargs.output
      else:
        output = os.path.splitext(os.path.basename(cliargs.input))[0]
      output = os.path.normpath(output)
      config.make_output_tree(output=output)
      presentation.save(config=config, output=output)

if __name__ == '__main__':
  main()



#!/usr/bin/env python
"""
figure.py, module definition of Figure class.
"""
import re
from yattag import Doc
from box import Box


class Figure(Box):
  """
  Object for handling figure-box. It is a subclass of Box.

  The syntax is:

  $figure
  $style[style_options]
  $caption[caption_options]{caption}
  $content[content_options]{content}
  $endfigure

  Note that differently from Box class:
  1. the "content_type" and "caption_type" are automatically set to "figure" and "Figure" respectively; anyhow they can be still
     specified inside the $figure/$endfigure environment;
  2. the "caption" is at bottom by default, but it can be also positionated at the top.

  Attributes
  ----------
  regexs: dict
    dictionary of regexs
  figures_number : int
    global number of figures (equals to the number of Figure instances)
  """
  regexs = {'figure': re.compile(r"(?P<figure>\$figure(?P<env>.*?)\$endfigure)", re.DOTALL)}
  figures_number = 0

  @classmethod
  def reset(cls):
    """Reset to default state."""
    cls.figures_number = 0
    return

  def __init__(self, source=None):
    """
    Parameters
    ----------
    source : str, optional
      string (as single stream) containing the source

    Attributes
    ----------
    number : int
      number of figure
    """
    super(Figure, self).__init__(ctn_type='figure')
    self.cap_type = 'Figure'
    Figure.figures_number += 1
    self.number = Figure.figures_number
    if source:
      self.get(source=source)
    return

  def to_html(self):
    """Convert self data to its html stream."""
    doc = Doc()
    with doc.tag('div', id='Figure-' + str(self.number)):
      if self.style:
        doc.attr(style=self.style)
      else:
        doc.attr(klass='figure')
      with doc.tag(self.ctn_type):
        if self.cap_position is not None and self.cap_position.upper() == 'TOP':
          self.put_caption(doc=doc, klass='figure-caption')
        if self.ctn_options:
          doc.stag('img', src=self.ctn, klass='figure-content', style=self.ctn_options, alt='Figure-' + self.ctn)
        else:
          doc.stag('img', src=self.ctn, klass='figure-content', style='width:100%;', alt='Figure-' + self.ctn)
        if self.cap_position is None or self.cap_position.upper() == 'BOTTOM':
          self.put_caption(doc=doc, klass='figure-caption')
    return doc.getvalue()



#!/usr/bin/env python
"""
note.py, module definition of Note class.
"""
import re
from yattag import Doc
from box import Box
from markdown_utils import markdown2html


class Note(Box):
  """
  Object for handling note-box. It is a subclass of Box.

  The syntax is:

  $note
  $style[style_options]
  $caption[caption_options]{caption}
  $content[content_options]{content}
  $endnote

  Note that differently from Box class:
  1. the "content_type" and "caption_type" are automatically set to "note" and "Note" respectively; anyhow they can be still
     specified inside the $note/$endnote environment;
  2. the "caption" is at top by default, but it can be also positionated at the bottom.

  Attributes
  ----------
  notes_number : int
    global number of notes (equals to the number of Note instances)
  """
  regexs = {'note': re.compile(r"(?P<note>\$note(?P<env>.*?)\$endnote)", re.DOTALL)}
  notes_number = 0

  @classmethod
  def reset(cls):
    """Reset to default state."""
    cls.notes_number = 0
    return

  def __init__(self, source=None):
    """
    Parameters
    ----------
    source : str, optional
      string (as single stream) containing the source

    Attributes
    ----------
    number : int
      note number
    """
    super(Note, self).__init__(ctn_type='note')
    self.cap_type = 'Note'
    Note.notes_number += 1
    self.number = Note.notes_number
    if source:
      self.get(source=source)
    return

  def to_html(self):
    """Convert self data to its html stream."""
    doc = Doc()
    with doc.tag('div', id='note-' + str(self.number)):
      if self.style:
        doc.attr(style=self.style)
      else:
        doc.attr(klass='note')
      if self.cap_position is None or self.cap_position.upper() == 'TOP':
        self.put_caption(doc=doc, klass='note-caption')
      with doc.tag('div', klass='note-content'):
        if self.ctn_options:
          doc.attr(style=self.ctn_options)
        doc.asis(markdown2html(self.ctn, no_p=True))
      if self.cap_position is not None and self.cap_position.upper() == 'BOTTOM':
        self.put_caption(doc=doc, klass='note-caption')
    return doc.getvalue()



#!/usr/bin/env python
"""
markdown_utils.py, module definition of markdown utils functions.
"""
import markdown
from mdx_mathjax import MathJaxExtension
try:
  from markdown_checklist.extension import ChecklistExtension
  __mdx_checklist__ = True
except ImportError:
  __mdx_checklist__ = False


def markdown2html(source, no_p=False):
  """Convert markdown source to html.

  Parameters
  ----------
  source : str
    string (as single stream) containing the source
  no_p : bool, optional
    if True the converted contents is not inserted into the <p></p> tags

  Returns
  -------
  str
    converted source
  """
  if __mdx_checklist__:
    mkd = markdown.Markdown(output_format='html5',
                            extensions=['smarty',
                                        'extra',
                                        ChecklistExtension(),
                                        MathJaxExtension()])
  else:
    mkd = markdown.Markdown(output_format='html5',
                            extensions=['smarty',
                                        'extra',
                                        MathJaxExtension()])
  markup = mkd.reset().convert(source)
  if no_p:
    p_start = '<p>'
    p_end = '</p>'
    if markup.startswith(p_start) and markup.endswith(p_end):
      markup = markup[len(p_start):-len(p_end)]
  return markup



#!/usr/bin/env python
"""
columns.py, module definition of Columns class.
"""
from __future__ import print_function
import re
from yattag import Doc
from markdown_utils import markdown2html


__recolumns__ = re.compile(r"(?P<columns>\$columns(?P<env>.*?)\$endcolumns)", re.DOTALL)
__recol__ = re.compile(r"(?P<column>\$column(\[(?P<options>.*?)\])*)", re.DOTALL)


class Columns(object):
  """
  Object for handling columns environment.

  The syntax is:

  $columns
  $column[width:30%;background:green;]
  the first column contents
  $column[width:60%;background:white;]
  the second column contents
  $column[width:10%;background:red;]
  the third column contents
  $endcolumns
  """
  regexs = {"columns": re.compile(r"(?P<columns>\$columns(?P<env>.*?)\$endcolumns)", re.DOTALL),
            "column": re.compile(r"(?P<column>\$column(\[(?P<options>.*?)\])*)", re.DOTALL)}

  def __init__(self, source=None):
    """
    Parameters
    ----------
    source : str, optional
      string (as single stream) containing the source

    Attributes
    ----------
    number : int
      columns number
    columns : list
      list of list containing contents and options [cont,opts] of columns
    """
    self.number = 0
    self.columns = []
    if source:
      self.get(source=source)
    return

  def __str__(self):
    string = []
    if self.number > 0:
      string.append('\nColumns number: ' + str(self.number))
      for col, column in enumerate(self.columns):
        string.append('\nColumn(' + str(col) + ')[options=' + str(column[1]) + ']')
    return ''.join(string)

  def get(self, source):
    """Get columns data.

    Parameters
    ----------
    source : str
      string (as single stream) containing the source
    """
    match = re.search(Columns.regexs['columns'], source)
    if match:
      source_columns = match.group('env')
      columns = []
      for match_col in re.finditer(Columns.regexs['column'], source_columns):
        columns.append([match_col.group('options'), match_col.start(), match_col.end()])
      if len(columns) > 0:
        for col, column in enumerate(columns):
          if col < len(columns) - 1:
            contents = source_columns[column[2] + 1:columns[col + 1][1]]
          else:
            contents = source_columns[column[2] + 1:]
          self.columns.append([contents, column[0]])
      self.number = len(self.columns)
    return

  def to_html(self):
    """Convert self data to its html stream."""
    if self.number > 0:
      doc = Doc()
      with doc.tag('div', klass='columns'):
        for col, column in enumerate(self.columns):
          with doc.tag('div', klass='column'):
            doc.attr(('column-number', str(col + 1)))
            style = 'display:block;float:left;'
            if column[1]:
              style += column[1]
            else:
              style += 'width:' + str(int(100.0 / self.number)) + '%;'
            doc.attr(style=style)
            doc.asis(markdown2html(source=column[0]))
      return doc.getvalue()
    return ''



#!/usr/bin/env python
"""
section.py, module definition of Section class.
"""
from __future__ import print_function
from collections import OrderedDict
from subsection import Subsection


class Section(object):
  """
  Section object.

  Attributes
  ----------
  subsections_number: int
  """
  subsections_number = 0

  @classmethod
  def reset(cls):
    """Reset to default state."""
    cls.subsections_number = 0
    Subsection.reset()
    return

  def __init__(self, number, title=None):
    self.number = number
    self.title = title
    self.subsections = []
    self.toc = OrderedDict()
    return

  def __str__(self):
    strings = [str(self.title)]
    for subsection in self.subsections:
      strings.append("    " + str(subsection))
    return '\n'.join(strings)

  def update_toc(self):
    """Update TOC after a new subsection (the last one) has been added."""
    self.toc[self.subsections[-1].title] = self.subsections[-1].toc

  def add_subsection(self, subsection):
    """
    Add a subsection to the section.

    Parameters
    ----------
    section: Subsection
    """
    Section.subsections_number += 1
    self.subsections.append(subsection)
    self.update_toc()
    return

  def put_html_attributes(self, doc):
    """Put html attibutes of the section.

    Parameters
    ----------
    doc: Doc
    """
    doc.attr(('sectionnumber', str(self.number)))
    if self.title is not None:
      doc.attr(('sectiontitle', str(self.title)))
    return



#!/usr/bin/env python
"""
metadata.py, module definition of Metadata class.
"""
from __future__ import print_function
import re
import sys
from yattag import Doc


class Metadata(object):
  """
  Object for handling metadata.
  """

  @classmethod
  def reset(cls):
    """Reset to default state."""
    return

  def __init__(self, name, value=None):
    """
    Parameters
    ----------
    name : str
    value : str|[str]
    """
    self.name = name
    self.value = value
    self.regex = re.compile(r"\$" + self.name + r"(\[(?P<style>.*?)\])*", re.DOTALL)
    return

  def update_value(self, value):
    """Update metadata value.

    value: str|[str]
    """
    if isinstance(self.value, list):
      if not isinstance(value, list):
        sys.stderr.write('Error: the value of metadata "' + self.name + '" must be a list!')
        sys.exit(1)
      self.value = []
      for val in value:
        self.value.append(str(val))
    else:
      self.value = str(value)

  def parse(self, parser, source, toc_depth=None, max_time=None, current=None):
    """Parse source for metadata placeholder substituting occurences with its value and its eventula css style.

    Parameters
    ----------
    parser: Parser
    source: str
    max_time: str
      max time for presentation
    """
    codeblocks = parser.tokenizer(source=source, re_search=parser.regexs['codeblock'])
    metadatablocks = parser.tokenizer(source=source, re_search=self.regex, exclude=codeblocks)
    if len(metadatablocks) > 0:
      parsed_source = source[:metadatablocks[0]['start']]
      for m, metadatablock in enumerate(metadatablocks[:-1]):
        parsed_source += self.to_html(match=metadatablock['match'], toc_depth=toc_depth, max_time=max_time, current=current) + source[metadatablock['end']:metadatablocks[m + 1]['start']]
      parsed_source += self.to_html(match=metadatablocks[-1]['match'], toc_depth=toc_depth, max_time=max_time, current=current) + source[metadatablocks[-1]['end']:]
      return parsed_source
    return source

  def logo_to_html(self, match):
    """Convert logo metadata to html stream.

    Parameters
    ----------
    match: re.match object

    Returns
    -------
    str:
      html stream
    """
    doc = Doc()
    style = None
    if match.group('style'):
      style = str(match.group('style'))
    if style is not None:
      doc.stag('img', src=self.value, alt=self.value, style=style)
    else:
      doc.stag('img', src=self.value, alt=self.value)
    return doc.getvalue()

  def timer_to_html(self, match, max_time):
    """Convert custom metadata to html stream.

    Parameters
    ----------
    match: re.match object
    max_time: str

    Returns
    -------
    str:
      html stream
    """
    doc = Doc()
    with doc.tag('span', klass='timercontainer'):
      style = None
      if match.group('style'):
        style = str(match.group('style'))
      if style:
        doc.attr(style=style)
      with doc.tag('div', klass='countDown'):
        with doc.tag('div'):
          doc.attr(klass='timer')
        if style:
          if 'controls' in style:
            with doc.tag('div', klass='timercontrols'):
              with doc.tag('input', type='button'):
                doc.attr(klass='btn reset', onclick='resetCountdown(' + str(max_time) + ');', value=' &#10227; ', title='reset')
              with doc.tag('input', type='button'):
                doc.attr(klass='btn stop', onclick='stopCountdown();', value=' &#9724; ', title='pause')
              with doc.tag('input', type='button'):
                doc.attr(klass='btn start', onclick='startCountdown();', value=' &#9654; ', title='start')
    return doc.getvalue().replace('amp;', '')

  @staticmethod
  def toc_put_text(doc, klass, current, text):
    """Put text of current TOC element.

    Parameters
    ----------
    doc: Doc()
    klass: str
    current: list
    text: str
    """
    if text != '':
      with doc.tag('span', klass=klass):
        doc.text('  ' * (len(current) - 1) + '.'.join(str(c) for c in current) + ' ' + text)

  @staticmethod
  def toc_put_slides(doc, subsection, depth, actual_current, current, numbering_start):
    """Put slides list into TOC.

    Parameters
    ----------
    doc: Doc()
    subsection: dict
      dictionary of slides contained into the subsection
    depth: int
    actual_current: list
    current: list
    numbering_start: int
    """
    for slide in subsection:
      actual_current[3] += 1
      if depth >= 4:
        with doc.tag('a', href='#slide-' + str(actual_current[4])):
          if current is not None and current[0:4] == actual_current[0:4]:
            Metadata.toc_put_text(doc=doc, klass='toc-slide-emph', current=actual_current[numbering_start:4], text=slide)
          else:
            Metadata.toc_put_text(doc=doc, klass='toc-slide', current=actual_current[numbering_start:4], text=slide)
      actual_current[4] += 1

  @staticmethod
  def toc_put_subsections(doc, section, depth, actual_current, current, numbering_start):
    """Put subsections list into TOC.

    Parameters
    ----------
    doc: Doc()
    section: dict
      dictionary of subsections contained into the section
    depth: int
    actual_current: list
    current: list
    numbering_start: int
    """
    for subsection in section:
      actual_current[2] += 1
      actual_current[3] = 0
      if depth >= 3:
        with doc.tag('a', href='#slide-' + str(actual_current[4])):
          if current is not None and current[0:3] == actual_current[0:3]:
            Metadata.toc_put_text(doc=doc, klass='toc-subsection-emph', current=actual_current[numbering_start:3], text=subsection)
          else:
            Metadata.toc_put_text(doc=doc, klass='toc-subsection', current=actual_current[numbering_start:3], text=subsection)
      if subsection == '' and len(section) == 1:
        next_numbering_start = numbering_start + 1
      else:
        next_numbering_start = numbering_start
      Metadata.toc_put_slides(doc=doc,
                              subsection=section[subsection],
                              depth=depth,
                              actual_current=actual_current,
                              current=current,
                              numbering_start=next_numbering_start)

  @staticmethod
  def toc_put_sections(doc, chapter, depth, actual_current, current, numbering_start):
    """Put sections list into TOC.

    Parameters
    ----------
    doc: Doc()
    chapter: dict
      dictionary of sections contained into the chapter
    depth: int
    actual_current: list
    current: list
    numbering_start: int
    """
    for section in chapter:
      actual_current[1] += 1
      actual_current[2] = 0
      actual_current[3] = 0
      if depth >= 2:
        with doc.tag('a', href='#slide-' + str(actual_current[4])):
          if current is not None and current[0:2] == actual_current[0:2]:
            Metadata.toc_put_text(doc=doc, klass='toc-section-emph', current=actual_current[numbering_start:2], text=section)
          else:
            Metadata.toc_put_text(doc=doc, klass='toc-section', current=actual_current[numbering_start:2], text=section)
      if section == '' and len(chapter) == 1:
        next_numbering_start = numbering_start + 1
      else:
        next_numbering_start = numbering_start
      Metadata.toc_put_subsections(doc=doc,
                                   section=chapter[section],
                                   depth=depth,
                                   actual_current=actual_current,
                                   current=current,
                                   numbering_start=next_numbering_start)

  def toc_put_chapters(self, doc, depth, actual_current, current):
    """Put chapters list into TOC.

    Parameters
    ----------
    doc: Doc()
    depth: int
    actual_current: list
    current: list
    """
    for chapter in self.value:
      actual_current[0] += 1
      actual_current[1] = 0
      actual_current[2] = 0
      actual_current[3] = 0
      with doc.tag('a', href='#slide-' + str(actual_current[4])):
        if current is not None and current[0] == actual_current[0]:
          self.toc_put_text(doc=doc, klass='toc-chapter-emph', current=actual_current[0:1], text=chapter)
        else:
          self.toc_put_text(doc=doc, klass='toc-chapter', current=actual_current[0:1], text=chapter)
      if chapter == '' and len(self.value) == 1:
        numbering_start = 1
      else:
        numbering_start = 0
      self.toc_put_sections(doc=doc,
                            chapter=self.value[chapter],
                            depth=depth,
                            actual_current=actual_current,
                            current=current,
                            numbering_start=numbering_start)

  def toc_to_html(self, match, current=None, depth=1):
    """Convert TOC to a plain string.

    Parameters
    ----------
    match: re.match()
    current: [current_chapter, current_section, current_subsection, current_slide]
      eventual current chpater-section-subsection-slide number
    depth: int
      depth of TOC: 4 => up-to slides, 3 => up-to subsections, 2 => up-to sections, 1 => only chapters

    Returns
    -------
    str:
      plain string of TOC
    """
    def get_style(match):
      """Get TOC style if one.

      Parameters
      ----------
      match: re.match()

      Returns
      -------
      str:
        style
      """
      style = None
      if match.group('style'):
        style = str(match.group('style'))
      return style

    def get_actual_depth(style, depth):
      """Get the actual depth of TOC using the eventual provided depth or the one defined into the style.

      Parameters
      ----------
      style: str
      depth: int
        depth of TOC: 4 => up-to slides, 3 => up-to subsections, 2 => up-to sections, 1 => only chapters

      Returns
      -------
      int:
        actual depth
      """
      actual_depth = int(depth)
      if style is not None:
        if 'depth' in style.lower():
          match_depth = re.search(r'depth\:(?P<depth>[1-4])\;*', style)
          if match_depth.group('depth'):
            actual_depth = int(match_depth.group('depth'))
      return actual_depth

    style = get_style(match=match)
    actual_depth = get_actual_depth(style=style, depth=depth)
    doc = Doc()
    # numbering: [local_chap, local_sec, local_subsec, local_slide, global_slide]
    actual_current = [0, 0, 0, 0, 1]
    with doc.tag('div', klass='toc'):
      if style is not None:
        doc.attr(style=style)
      self.toc_put_chapters(doc=doc,
                            depth=actual_depth,
                            actual_current=actual_current,
                            current=current)
    return '\n' + doc.getvalue()

  def custom_to_html(self, match):
    """Convert custom metadata to html stream.

    Parameters
    ----------
    match: re.match object

    Returns
    -------
    str:
      html stream
    """
    doc = Doc()
    with doc.tag('span', klass='metadata'):
      style = None
      if match.group('style'):
        style = str(match.group('style'))
      if style:
        doc.attr(style=style)
        doc.asis(re.search(r'value\:(?P<value>.*?)\;', style).group('value'))
    return doc.getvalue()

  def to_html(self, match, toc_depth=None, max_time=None, current=None):
    """Convert logo metadata to html stream.

    Parameters
    ----------
    match: re.match object
    max_time: str
      max time for presentation

    Returns
    -------
    str:
      html stream
    """
    if 'toc' in self.name:
      return self.toc_to_html(match=match, current=current, depth=int(toc_depth))
    if 'logo' in self.name:
      return self.logo_to_html(match)
    elif 'timer' in self.name:
      return self.timer_to_html(match=match, max_time=max_time)
    elif 'custom' in self.name:
      return self.custom_to_html(match)
    else:
      doc = Doc()
      with doc.tag('span', klass='metadata'):
        style = None
        if match.group('style'):
          style = str(match.group('style'))
        if style:
          doc.attr(style=style)
        if isinstance(self.value, list):
          doc.asis(', '.join(self.value))
        else:
          doc.asis(str(self.value))
    return doc.getvalue()



#!/usr/bin/env python
"""
position.py, module definition of Position class.
"""
from __future__ import print_function


class Position(object):
  """
  Object for handling slide position into the infinite canvas.
  """

  @classmethod
  def reset(cls):
    """Reset to default state."""

  def __init__(self):
    """
    Attributes
    ----------
    """
    self.position = {'x': 0, 'y': 0, 'z': 0,
                     'rotx': 0, 'roty': 0, 'rotz': 0,
                     'scale': 1}

  def __update_position_absolute(self, transition):
    """Update position for absolute transition."""
    if transition['data-x'] != '':
      self.position['x'] = int(transition['data-x'])
    if transition['data-y'] != '':
      self.position['y'] = int(transition['data-y'])
    if transition['data-z'] != '':
      self.position['z'] = int(transition['data-z'])
    if transition['data-rotate-x'] != '':
      self.position['rotx'] = int(transition['data-rotate-x'])
    if transition['data-rotate-y'] != '':
      self.position['roty'] = int(transition['data-rotate-y'])
    if transition['data-rotate-z'] != '':
      self.position['rotz'] = int(transition['data-rotate-z'])
    if transition['scale'] != '':
      self.position['scale'] = int(transition['scale'])

  def __update_position_svgpath(self, transition):
    """Update position for svgpath transition."""

  def __update_position_horizontal(self, transition):
    """Update position for horizontal transition."""
    self.position['x'] = self.position['x'] + transition['width'] * (max(self.position['scale'], transition['scale']) + transition['offset'] / 100.0)

  def __update_position_neg_horizontal(self, transition):
    """Update position for -horizontal transition."""
    self.position['x'] = self.position['x'] - transition['width'] * (max(self.position['scale'], transition['scale']) + transition['offset'] / 100.0)

  def __update_position_vertical(self, transition):
    """Update position for vertical transition."""
    self.position['y'] = self.position['y'] + transition['height'] * (max(self.position['scale'], transition['scale']) + transition['offset'] / 100.0)

  def __update_position_neg_vertical(self, transition):
    """Update position for -vertical transition."""
    self.position['y'] = self.position['y'] - transition['height'] * (max(self.position['scale'], transition['scale']) + transition['offset'] / 100.0)

  def __update_position_diagonal(self, transition):
    """Update position for diagonal transition."""
    self.position['x'] = self.position['x'] + transition['width'] * (max(self.position['scale'], transition['scale']) + transition['offset'] / 100.0)
    self.position['y'] = self.position['y'] + transition['height'] * (max(self.position['scale'], transition['scale']) + transition['offset'] / 100.0)

  def __update_position_neg_diagonal(self, transition):
    """Update position for -diagonal transition."""
    self.position['x'] = self.position['x'] - transition['width'] * (max(self.position['scale'], transition['scale']) + transition['offset'] / 100.0)
    self.position['y'] = self.position['y'] - transition['height'] * (max(self.position['scale'], transition['scale']) + transition['offset'] / 100.0)

  def __update_position_diagonal_neg_x(self, transition):
    """Update position for diagonal-x transition."""
    self.position['x'] = self.position['x'] - transition['width'] * (max(self.position['scale'], transition['scale']) + transition['offset'] / 100.0)
    self.position['y'] = self.position['y'] + transition['height'] * (max(self.position['scale'], transition['scale']) + transition['offset'] / 100.0)

  def __update_position_diagonal_neg_y(self, transition):
    """Update position for diagonal-y transition."""
    self.position['x'] = self.position['x'] + transition['width'] * (max(self.position['scale'], transition['scale']) + transition['offset'] / 100.0)
    self.position['y'] = self.position['y'] - transition['height'] * (max(self.position['scale'], transition['scale']) + transition['offset'] / 100.0)

  def update_position(self, presentation_theme, overtheme=None):
    """Update the position using theme/overtheme slides-transition data.

    Parameters
    ----------
    presentation_theme: Theme()
      main presentation theme
    overtheme: Theme()
      eventual slide overtheme
    """
    update = {'absolute': self.__update_position_absolute,
              'svgpath': self.__update_position_svgpath,
              'horizontal': self.__update_position_horizontal,
              '-horizontal': self.__update_position_neg_horizontal,
              'vertical': self.__update_position_vertical,
              '-vertical': self.__update_position_neg_vertical,
              'diagonal': self.__update_position_diagonal,
              '-diagonal': self.__update_position_neg_diagonal,
              'diagonal-x': self.__update_position_diagonal_neg_x,
              'diagonal-y': self.__update_position_diagonal_neg_y}
    if overtheme is not None and overtheme.custom:
      theme = overtheme
    else:
      theme = presentation_theme

    transition = theme.get_slide_transition()
    if transition['transition'].lower() in update:
      update[transition['transition'].lower()](transition=transition)
    else:
      print('Warning: the slide transition "' + transition['transition'] + '" is unknown!')
      update['horizontal'](transition=transition)



#!/usr/bin/env python
"""
slide.py, module definition of Parser class.
This defines the MaTiSSe.py Parser.
"""
from __future__ import print_function
import os
import re
import sys


class Parser(object):
  """
  Parser of MaTiSSe.py sources.
  """
  regexs = {"all": re.compile(r"(.*(?P<expr>))", re.DOTALL)}

  def __init__(self):
    """
    Parameters
    ----------

    Attributes
    ----------
    regexs: dict
      dictionary filled with compiled objects of MaTiSSe.py regexs
    """
    self.regexs = {"chapter": re.compile(r"([^#]#|^#)\s+" + r"(?P<expr>.*)"),
                   "section": re.compile(r"([^#]##|^##)\s+" + r"(?P<expr>.*)"),
                   "subsection": re.compile(r"([^#]###|^###)\s+" + r"(?P<expr>.*)"),
                   "slide": re.compile(r"([^#]####|^####)\s+" + r"(?P<expr>.*)"),
                   "codeblock": re.compile(r"(?P<block>[`]{3}.*?[`]{3})", re.DOTALL),
                   "code": re.compile(r"(?P<block>`.*?`)"),
                   "yamlblock": re.compile(r"(?P<block>[-]{3}.*?[-]{3})", re.DOTALL),
                   "includeblock": re.compile(r"\$include\((?P<include>.*?)\)")}
    return

  @staticmethod
  def tokenizer(source, re_search, exclude=None, force_all=False):
    """Tokenize accordingly to re_search (and exlude if passed).

    Parameters
    ----------
    source: str
      input stream
    re_search: compiled regex
    exclude: list
      list of start/end index of source lines

    Returns
    -------
    tokens: list
      list of token
    """
    def __tokenizer(source, re_search, exclude=None):
      tokens = []
      for match in re.finditer(re_search, source):
        safe = True
        if exclude is not None:
          for exc in exclude:
            if match.start() >= exc['start'] and match.end() <= exc['end']:
              safe = False
        if safe:
          tokens.append({'match': match, 'start': match.start(), 'end': match.end()})
      return tokens

    tokens = __tokenizer(source=source, re_search=re_search, exclude=exclude)
    if len(tokens) == 0 and force_all:
      tokens = __tokenizer(source=source, re_search=Parser.regexs['all'], exclude=exclude)
      return tokens[:-1]
    return tokens

  @staticmethod
  def tokens_end_update(tokens, end=None):
    """Update the end of tokens accordinly to the start of the next one into the tokens list.

    Paramaters
    ----------
    tokens: list

    Returns
    -------
    tokens: list
    """
    if len(tokens) > 0:
      for k, tok in enumerate(tokens[:-1]):
        tok['end_next'] = tokens[k + 1]['start']
      if end is not None:
        tokens[-1]['end_next'] = end
    return tokens

  @staticmethod
  def slides_end_update(slides, others):
    """Update the end of slides accordinly to the start of others sectionings.

    Paramaters
    ----------
    slides: list
    others: list

    Returns
    -------
    slides: list
    """
    for slide in slides:
      for other in others:
        if slide['start'] < other['start'] and slide['end_next'] >= other['start']:
          slide['end_next'] = other['start']
    return slides

  def includes(self, source):
    """
    Recursive inclusion of other sources.

    Parameters
    ----------
    source: str
      input stream

    Returns
    -------
    str:
      included sources stream
    """
    codeblocks = self.tokenizer(source=source, re_search=self.regexs['codeblock'])
    includeblocks = self.tokenizer(source=source, re_search=self.regexs['includeblock'], exclude=codeblocks)
    if len(includeblocks) > 0:
      complete_source = source[:includeblocks[0]['start']]
      for block, includeblock in enumerate(includeblocks[:-1]):
        include_file = includeblock['match'].group('include')
        if not os.path.exists(include_file):
          sys.stderr.write('Error: cannot include "' + include_file + '"')
          sys.exit(1)
        else:
          with open(include_file, 'r') as inc:
            other_source = inc.read()
          complete_source += other_source + source[includeblock['end']:includeblocks[block + 1]['start']]
      include_file = includeblocks[-1]['match'].group('include')
      if not os.path.exists(include_file):
        sys.stderr.write('Error: cannot include "' + include_file + '"')
        sys.exit(1)
      else:
        with open(include_file, 'r') as inc:
          other_source = inc.read()
      complete_source += other_source + source[includeblocks[-1]['end']:]
      source = complete_source
      source = self.includes(source=source)
    return source

  def tokenize(self, source):
    """Tokenize input source returning tagged tokens.

    Parameters
    ----------
    source: str
      input stream

    Returns
    -------
    tokens: dict
      dictionary of tokens; each element is a list of type [token, start_char, end_char]
    """
    codeblocks = self.tokenizer(source=source, re_search=self.regexs['codeblock'])
    yamlblocks = self.tokenizer(source=source, re_search=self.regexs['yamlblock'], exclude=codeblocks)
    chapters = self.tokenizer(source=source, re_search=self.regexs['chapter'], exclude=codeblocks + yamlblocks, force_all=True)
    chapters = self.tokens_end_update(tokens=chapters, end=len(source))
    sections = self.tokenizer(source=source, re_search=self.regexs['section'], exclude=codeblocks + yamlblocks, force_all=True)
    sections = self.tokens_end_update(tokens=sections, end=len(source))
    subsections = self.tokenizer(source=source, re_search=self.regexs['subsection'], exclude=codeblocks + yamlblocks, force_all=True)
    subsections = self.tokens_end_update(tokens=subsections, end=len(source))
    slides = self.tokenizer(source=source, re_search=self.regexs['slide'], exclude=codeblocks + yamlblocks)
    slides = self.tokens_end_update(tokens=slides, end=len(source))
    slides = self.slides_end_update(slides=slides, others=chapters)
    slides = self.slides_end_update(slides=slides, others=sections)
    slides = self.slides_end_update(slides=slides, others=subsections)
    tokens = {'codeblocks': codeblocks,
              'yamlblocks': yamlblocks,
              'chapters': chapters,
              'sections': sections,
              'subsections': subsections,
              'slides': slides}
    return tokens



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
theme.py, module definition of Theme class.
"""
from __future__ import print_function
from copy import deepcopy
from collections import OrderedDict
from yaml import load_all, YAMLError
from box import Box
from note import Note
from figure import Figure
from table import Table
from video import Video


__css__ = """/* CSS theme generated by MaTiSSe.py */
*, *:after, *:before {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  margin: 0;
  padding: 0;
}
p {
  padding: 1% 0;
}
a {
  color: inherit;
  text-decoration: none;
  transition: 0.5s;
}
a:hover, a:focus {
  background: rgba(255,255,255,1);
  text-shadow: -1px -1px 2px rgba(100,100,100,0.5);
}
ol {
  list-style: none;
  counter-reset: item;
  padding-left: 0;
}
ul {
  list-style: none;
  padding-left: 0;
}
li:before {
  content: counter(item) ". ";
  counter-increment: item;
  padding-right: 0.8em;
  text-indent: -0.8em;
}
ul li:before {
  content: '■';
}
ol li:before {
  padding-right: 0.5em;
}
li {
  padding-left: 0.8em;
}
pre {
  display:flex;
  padding: 1% 0;
  font-size: 90%;
}
.columns:after {
  visibility: hidden;
  content:" ";
  clear: both;
 display: block;
}
input[type=button] {
  padding:0;
  margin:0;
  border:0 none;
  background: rgba(0,0,0,0.01);
}
.countDown {
  display: block;
  padding: 0;
  margin: 0;
}
.timercontrols {
  display:table-row;
}
.timer {
  display:table-row;
}
.timercontainer {
  padding: 0% 1%;
}
.step {
  position: relative;
}
.impress-enabled .step.active {
  opacity: 1;
}
.impress-enabled .step {
  margin: 0;
  opacity: 0.3;
  -webkit-transition: opacity 1s;
  -moz-transition: opacity 1s;
  -ms-transition: opacity 1s;
  -o-transition: opacity 1s;
  transition: opacity 1s;
}
.toc {
  display: block;
}
.toc-chapter, .toc-section, .toc-subsection, .toc-slide {
  clear: both;
  float: left;
  white-space: pre-wrap;
}
.toc-chapter-emph, .toc-section-emph, .toc-subsection-emph, .toc-slide-emph {
  clear: both;
  float: left;
  white-space: pre-wrap;
}
"""


class Theme(object):
  """
  Presentation theme object.
  """

  @classmethod
  def reset(cls):
    """Reset to default state."""
    Box.reset()
    Note.reset()
    Figure.reset()
    Table.reset()
    Video.reset()

  def __init__(self, source=None, name='theme', div_id='', default=True):
    """
    Parameters
    ----------
    source: str
    name: str
    div_id: str
    default: bool
      add defaults css

    Attributes
    ----------
    canvas = dict
      dictionary of canvas theme
    toc = dict
      dictionary of table of contents theme
    slide = dict
      dictionary of slide theme
    css = str
      css theme as single stream
    """
    self.copy_from_theme = None
    self.canvas = []
    self.ordered_list_items = []
    self.unordered_list_items = []
    self.toc = []
    self.toc_chapter_emph = []
    self.toc_section_emph = []
    self.toc_subsection_emph = []
    self.toc_slide_emph = []
    self.slide = []
    self.slide_content = []
    self.slide_header = {}
    self.slide_footer = {}
    self.slide_sidebar = {}
    self.box = []
    self.box_caption = []
    self.box_content = []
    self.note = []
    self.note_caption = []
    self.note_content = []
    self.table = []
    self.table_caption = []
    self.table_content = []
    self.figure = []
    self.figure_caption = []
    self.figure_content = []
    self.video = []
    self.video_caption = []
    self.video_content = []
    self.slide_header_metadata = {}
    self.slide_footer_metadata = {}
    self.slide_sidebar_metadata = {}
    self.css = None
    self.custom = False
    self.div_id = div_id
    self.default = default
    if source is not None:
      self.get(source=source, name=name, div_id=div_id, default=default)
    return

  def __str__(self):
    if self.css is None:
      return ''
    return self.css

  def set_from(self, other):
    """Set self attributes deepcopying theme from other theme.

    Parameters
    ----------
    other: Theme()
    """
    self.canvas = deepcopy(other.canvas)
    self.toc = deepcopy(other.toc)
    self.toc_chapter_emph = deepcopy(other.toc_chapter_emph)
    self.toc_section_emph = deepcopy(other.toc_section_emph)
    self.toc_subsection_emph = deepcopy(other.toc_subsection_emph)
    self.toc_slide_emph = deepcopy(other.toc_slide_emph)
    self.slide = deepcopy(other.slide)
    self.slide_content = deepcopy(other.slide_content)
    self.slide_header = deepcopy(other.slide_header)
    self.slide_footer = deepcopy(other.slide_footer)
    self.slide_sidebar = deepcopy(other.slide_sidebar)
    self.box = deepcopy(other.box)
    self.box_caption = deepcopy(other.box_caption)
    self.box_content = deepcopy(other.box_content)
    self.note = deepcopy(other.note)
    self.note_caption = deepcopy(other.note_caption)
    self.note_content = deepcopy(other.note_content)
    self.table = deepcopy(other.table)
    self.table_caption = deepcopy(other.table_caption)
    self.table_content = deepcopy(other.table_content)
    self.figure = deepcopy(other.figure)
    self.figure_caption = deepcopy(other.figure_caption)
    self.figure_content = deepcopy(other.figure_content)
    self.video = deepcopy(other.video)
    self.video_caption = deepcopy(other.video_caption)
    self.video_content = deepcopy(other.video_content)
    self.slide_header_metadata = deepcopy(other.slide_header_metadata)
    self.slide_footer_metadata = deepcopy(other.slide_footer_metadata)
    self.slide_sidebar_metadata = deepcopy(other.slide_sidebar_metadata)
    self.css = deepcopy(other.css)
    self.custom = deepcopy(other.custom)

  def copy_from(self, other):
    """Copy attributes from other theme if self has not already set those attributes.

    Parameters
    ----------
    other: Theme()
    """
    def append_css(my_element, other_element):
      if len(other_element) > 0:
        for css in other_element:
          found = False
          for mycss in my_element:
            if isinstance(css, dict):
              if css.keys()[0] in mycss:
                found = True
            elif css in mycss:
              found = True
          if not found:
            if isinstance(my_element, list):
              my_element.append(css)
            else:
              my_element[css] = other_element[css]

    append_css(my_element=self.canvas, other_element=other.canvas)
    append_css(my_element=self.toc, other_element=other.toc)
    append_css(my_element=self.toc_chapter_emph, other_element=other.toc_chapter_emph)
    append_css(my_element=self.toc_section_emph, other_element=other.toc_section_emph)
    append_css(my_element=self.toc_subsection_emph, other_element=other.toc_subsection_emph)
    append_css(my_element=self.toc_slide_emph, other_element=other.toc_slide_emph)
    append_css(my_element=self.slide, other_element=other.slide)
    append_css(my_element=self.slide_content, other_element=other.slide_content)
    for header in other.slide_header:
      if header not in self.slide_header:
        self.slide_header[header] = other.slide_header[header]
      else:
        append_css(my_element=self.slide_header[header], other_element=other.slide_header[header])
    for footer in other.slide_footer:
      if footer not in self.slide_footer:
        self.slide_footer[footer] = other.slide_footer[footer]
      else:
        append_css(my_element=self.slide_footer[footer], other_element=other.slide_footer[footer])
    for sidebar in other.slide_sidebar:
      if sidebar not in self.slide_sidebar:
        self.slide_sidebar[sidebar] = other.slide_sidebar[sidebar]
      else:
        append_css(my_element=self.slide_sidebar[sidebar], other_element=other.slide_sidebar[sidebar])
    append_css(my_element=self.box, other_element=other.box)
    append_css(my_element=self.box_caption, other_element=other.box_caption)
    append_css(my_element=self.box_content, other_element=other.box_content)
    append_css(my_element=self.note, other_element=other.note)
    append_css(my_element=self.note_caption, other_element=other.note_caption)
    append_css(my_element=self.note_content, other_element=other.note_content)
    append_css(my_element=self.table, other_element=other.table)
    append_css(my_element=self.table_caption, other_element=other.table_caption)
    append_css(my_element=self.table_content, other_element=other.table_content)
    append_css(my_element=self.figure, other_element=other.figure)
    append_css(my_element=self.figure_caption, other_element=other.figure_caption)
    append_css(my_element=self.figure_content, other_element=other.figure_content)
    append_css(my_element=self.video, other_element=other.video)
    append_css(my_element=self.video_caption, other_element=other.video_caption)
    append_css(my_element=self.video_content, other_element=other.video_content)
    for header in other.slide_header_metadata:
      if header not in self.slide_header_metadata:
        self.slide_header_metadata[header] = other.slide_header_metadata[header]
      else:
        append_css(my_element=self.slide_header_metadata[header], other_element=other.slide_header_metadata[header])
    for footer in other.slide_footer_metadata:
      if footer not in self.slide_footer_metadata:
        self.slide_footer_metadata[footer] = other.slide_footer_metadata[footer]
      else:
        append_css(my_element=self.slide_footer_metadata[footer], other_element=other.slide_footer_metadata[footer])
    for sidebar in other.slide_sidebar_metadata:
      if sidebar not in self.slide_sidebar_metadata:
        self.slide_sidebar_metadata[sidebar] = other.slide_sidebar_metadata[sidebar]
      else:
        append_css(my_element=self.slide_sidebar_metadata[sidebar], other_element=other.slide_sidebar_metadata[sidebar])
    self.__get_css()

  @staticmethod
  def theme2css(div, div_id, klass, theme_list):
    """
    Convert theme list data to css.

    Parameters
    ----------
    div: str
      div tag type
    div_id: str
    klass: str
      css class name
    theme_list: list
      theme list data

    Returns
    -------
    css: str
    """
    if len(theme_list) > 0:
      head = ''
      if div_id != '':
        head += '#' + div_id
      if div != '':
        head += ' ' + div + ' '
      if klass != '':
        if klass == 'slide':
          head += '.' + klass + ' '
        else:
          head += ' .' + klass + ' '
      css = ['\n' + head + '{']
      for element in theme_list:
        for key in element:
          if 'metadata' not in key.lower():
            css.append('\n  ' + key + ': ' + element[key] + ';')
      css.append('\n}')
      return ''.join(css)
    return ''

  @staticmethod
  def check_css_attribute(name, value, css_list, retain=False):
    """Ensure css attribute presence into CSS theme.

    Parameters
    ----------
    name: str
      attribute name
    value: str
      attribute value
    css_list: list
    retain: bool
      retain the already present value
    """
    found = False
    for css in css_list:
      if found:
        break
      for key in css:
        if name.lower() in key.lower():
          if not retain:
            css[key] = value
          found = True
          break
    if not found:
      css_list.append({name: value})

  def __get_slide_decorators_metadata(self):
    """Get metadata of slide decorators (headers, footers, sidebars)."""
    def __get_decorators(decorator):
      decorators = getattr(self, 'slide_' + decorator)
      for decor in decorators:
        getattr(self, 'slide_' + decorator + '_metadata')[decor] = OrderedDict()
        for data in decorators[decor]:
          for key in data:
            if 'metadata' in key.lower():
              for meta in data[key]:
                for meta_key in meta:
                  css = ''
                  for elem in meta[meta_key]:
                    for elem_key in elem:
                      css += elem_key + ':' + elem[elem_key] + ';'
                  getattr(self, 'slide_' + decorator + '_metadata')[decor][meta_key] = css

    __get_decorators(decorator='header')
    __get_decorators(decorator='footer')
    __get_decorators(decorator='sidebar')

  def __get_copy_from(self, data):
    """
    Get copy-from value.

    Parameters
    ----------
    data: dict
    """
    if 'copy-from-theme' in data:
      self.copy_from_theme = data['copy-from-theme']

  def __get_canvas(self, data):
    """
    Get canvas theme.

    Parameters
    ----------
    data: dict
    """
    if 'canvas' in data:
      self.canvas = data['canvas']

  def __get_ordered_list_items(self, data):
    """
    Get ordered-list-items theme.

    Parameters
    ----------
    data: dict
    """
    if 'ordered-list-items' in data:
      self.ordered_list_items = data['ordered-list-items']

  def __get_unordered_list_items(self, data):
    """
    Get unordered-list-items theme.

    Parameters
    ----------
    data: dict
    """
    if 'unordered-list-items' in data:
      self.unordered_list_items = data['unordered-list-items']

  def __get_toc(self, data):
    """
    Get toc theme.

    Parameters
    ----------
    data: dict
    """
    if 'toc' in data:
      toc = data['toc']
      for toc_element in toc:
        for key in toc_element:
          if 'emph' not in key:
            self.toc.append(toc_element)
          if 'chapter-emph' in key:
            self.toc_chapter_emph = toc_element[key]
          if 'section-emph' in key and 'subsection' not in key:
            self.toc_section_emph = toc_element[key]
          if 'subsection-emph' in key:
            self.toc_subsection_emph = toc_element[key]
          if 'slide-emph' in key:
            self.toc_slide_emph = toc_element[key]

  def __check_slide_dimensions(self):
    """Ensure dimensions assignment for slide theme."""
    found_w = False
    found_h = False
    for css in self.slide:
      if found_w and found_h:
        break
      for key in css:
        if 'width' in key.lower():
          found_w = True
        if 'height' in key.lower():
          found_h = True
    if not found_w:
      self.slide.append({'width': '900px'})
    if not found_h:
      self.slide.append({'height': '700px'})

  def __check_slide_headers_dimensions(self):
    """Ensure dimensions assignment for slide headers theme."""
    for header in self.slide_header:
      found_w = False
      found_h = False
      for css in self.slide_header[header]:
        for key in css:
          if 'width' in key.lower():
            found_w = True
          if 'height' in key.lower():
            found_h = True
      if not found_w:
        self.slide_header[header].append({'width': '100%'})
      if not found_h:
        self.slide_header[header].append({'height': '10%'})

  def __check_slide_footers_dimensions(self):
    """Ensure dimensions assignment for slide footers theme."""
    for footer in self.slide_footer:
      found_w = False
      found_h = False
      for css in self.slide_footer[footer]:
        for key in css:
          if 'width' in key.lower():
            found_w = True
          if 'height' in key.lower():
            found_h = True
      if not found_w:
        self.slide_footer[footer].append({'width': '100%'})
      if not found_h:
        self.slide_footer[footer].append({'height': '10%'})

  @staticmethod
  def __check_active(decorator):
    """Check if a decorator is active.

    Parameters
    ----------
    decorator: list
      list of css attributes

    Returns
    -------
    bool:
      active status
    """
    active = False
    for css in decorator:
      for key in css:
        if 'active' in key.lower():
          active = css[key].lower() == 'yes'
    return active

  @staticmethod
  def __get_attribute(attribute, css_list):
    """Get css attribute from css_list.

    Parameters
    ----------
    attribute: str
      attribute name
    css_list: list
      list of css attributes

    Returns
    -------
    str:
      attribute value
    """
    for css in css_list:
      for key in css:
        if attribute in key.lower():
          return css[key]
    return ''

  def __get_slide_content_height(self):
    """Get the height of the slide content from the headers and footers remainder height.

    Returns
    -------
    height: str
    """
    height = 0
    for header in self.slide_header:
      active = self.__check_active(decorator=self.slide_header[header])
      if active:
        height += int(self.__get_attribute(attribute='height', css_list=self.slide_header[header]).strip('%'))
    for footer in self.slide_footer:
      active = self.__check_active(decorator=self.slide_footer[footer])
      if active:
        height += int(self.__get_attribute(attribute='height', css_list=self.slide_footer[footer]).strip('%'))
    return str(100 - height) + '%'

  def __get_slide_content_width(self):
    """Get the width of the slide content from the sidebars remainder width.

    Returns
    -------
    width: str
    """
    width = 0
    for sidebar in self.slide_sidebar:
      active = self.__check_active(decorator=self.slide_sidebar[sidebar])
      if active:
        width += int(self.__get_attribute(attribute='width', css_list=self.slide_sidebar[sidebar]).strip('%'))
      # for css in self.slide_sidebar[sidebar]:
      #   active = False
      #   for key in css:
      #     if 'active' in key.lower():
      #       active = css[key].lower() == 'yes'
      #       break
      #   if active:
      #     for key in css:
      #       if 'width' in key.lower():
      #         width += int(css[key].strip('%'))
      #         break
    return str(100 - width) + '%'

  def __check_slide_sidebars_dimensions(self):
    """Ensure dimensions assignment for slide sidebars theme."""
    height = self.__get_slide_content_height()
    for sidebar in self.slide_sidebar:
      found_w = False
      found_h = False
      for css in self.slide_sidebar[sidebar]:
        for key in css:
          if 'width' in key.lower():
            found_w = True
          if 'height' in key.lower():
            # css[key] = height
            found_h = True
      if not found_w:
        self.slide_sidebar[sidebar].append({'width': '10%'})
      if not found_h:
        self.slide_sidebar[sidebar].append({'height': height})

  def __check_slide_content_dimensions(self):
    """Ensure dimensions assignment for slide content theme."""
    height = self.__get_slide_content_height()
    width = self.__get_slide_content_width()
    found_w = False
    found_h = False
    for css in self.slide_content:
      for key in css:
        if 'width' in key.lower():
          css[key] = width
          found_w = True
        if 'height' in key.lower():
          css[key] = height
          found_h = True
    if not found_w:
      self.slide_content.append({'width': width})
    if not found_h:
      self.slide_content.append({'height': height})

  def __check_slide(self):
    """Chack slide consistency."""
    self.check_css_attribute(name='display', value='block', css_list=self.slide)
    self.check_css_attribute(name='display', value='block', css_list=self.slide_content)
    self.check_css_attribute(name='float', value='left', css_list=self.slide_content)
    for header in self.slide_header:
      self.check_css_attribute(name='display', value='block', css_list=self.slide_header[header])
      self.check_css_attribute(name='active', value='yes', css_list=self.slide_header[header], retain=True)
    for footer in self.slide_footer:
      self.check_css_attribute(name='display', value='block', css_list=self.slide_footer[footer])
      self.check_css_attribute(name='clear', value='both', css_list=self.slide_footer[footer])
      self.check_css_attribute(name='active', value='yes', css_list=self.slide_footer[footer], retain=True)
    for sidebar in self.slide_sidebar:
      self.check_css_attribute(name='display', value='block', css_list=self.slide_sidebar[sidebar])
      self.check_css_attribute(name='float', value='left', css_list=self.slide_sidebar[sidebar])
      self.check_css_attribute(name='position', value='R', css_list=self.slide_sidebar[sidebar], retain=True)
      self.check_css_attribute(name='active', value='yes', css_list=self.slide_sidebar[sidebar], retain=True)
    self.__check_slide_dimensions()
    self.__check_slide_headers_dimensions()
    self.__check_slide_footers_dimensions()
    self.__check_slide_sidebars_dimensions()
    self.__check_slide_content_dimensions()

  def __get_slide(self, data):
    """
    Get slide theme.

    Parameters
    ----------
    data: dict
    """
    if 'slide' in data:
      slide = data['slide']
      if slide is not None:
        for slide_element in slide:
          for key in slide_element:
            if key is not None:
              if 'content' not in key and 'header' not in key and 'footer' not in key and 'sidebar' not in key:
                self.slide.append(slide_element)
              if 'content' in key:
                self.slide_content = slide_element[key]
              if 'header' in key:
                self.slide_header[key] = slide_element[key]
              if 'footer' in key:
                self.slide_footer[key] = slide_element[key]
              if 'sidebar' in key:
                self.slide_sidebar[key] = slide_element[key]
      self.__check_slide()
      self.__get_slide_decorators_metadata()

  def __get_box_like(self, env, data):
    """
    Get box-like theme (box, note, table, figure, video).

    Parameters
    ----------
    data: dict
    """
    if env in data:
      env_data = data[env]
      for env_element in env_data:
        for key in env_element:
          if 'caption' not in key.lower() and 'content' not in key.lower():
            getattr(self, env).append(env_element)
          if 'caption' in key.lower():
            setattr(self, env + '_caption', env_element[key])
          if 'content' in key.lower():
            setattr(self, env + '_content', env_element[key])

  def __get_css(self):
    """Construct CSS theme."""
    if self.default:
      css = [__css__]
    else:
      css = []
    css.append(self.theme2css(div='body', div_id=self.div_id, klass='', theme_list=self.canvas))
    css.append(self.theme2css(div='ol li:before', div_id=self.div_id, klass='', theme_list=self.ordered_list_items))
    css.append(self.theme2css(div='ul li:before', div_id=self.div_id, klass='', theme_list=self.unordered_list_items))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='toc', theme_list=self.toc))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='toc-chapter-emph', theme_list=self.toc_chapter_emph))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='toc-section-emph', theme_list=self.toc_section_emph))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='toc-subsection-emph', theme_list=self.toc_subsection_emph))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='toc-slide-emph', theme_list=self.toc_slide_emph))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='slide', theme_list=self.slide))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='slide-content', theme_list=self.slide_content))
    for header in self.slide_header:
      css.append(self.theme2css(div='', div_id=self.div_id, klass='slide-' + header, theme_list=self.slide_header[header]))
    for footer in self.slide_footer:
      css.append(self.theme2css(div='', div_id=self.div_id, klass='slide-' + footer, theme_list=self.slide_footer[footer]))
    for sidebar in self.slide_sidebar:
      css.append(self.theme2css(div='', div_id=self.div_id, klass='slide-' + sidebar, theme_list=self.slide_sidebar[sidebar]))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='box', theme_list=self.box))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='box-caption', theme_list=self.box_caption))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='box-content', theme_list=self.box_content))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='note', theme_list=self.note))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='note-caption', theme_list=self.note_caption))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='note-content', theme_list=self.note_content))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='table', theme_list=self.table))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='table-caption', theme_list=self.table_caption))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='table-content', theme_list=self.table_content))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='figure', theme_list=self.figure))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='figure-caption', theme_list=self.figure_caption))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='figure-content', theme_list=self.figure_content))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='video', theme_list=self.video))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='video-caption', theme_list=self.video_caption))
    css.append(self.theme2css(div='', div_id=self.div_id, klass='video-content', theme_list=self.video_content))
    self.css = ''.join(css)

  def get(self, source, name='theme', div_id='', default=True):
    """
    Get theme from source stream.

    Parameters
    ----------
    source: str
    name: str
    div_id: str
    default: bool
      add defaults css
    """
    self.div_id = div_id
    self.default = default
    if len(source) > 0:
      try:
        for data in load_all(source):
          if name in data:
            for element in data[name]:
              self.__get_copy_from(data=element)
              self.__get_canvas(data=element)
              self.__get_ordered_list_items(data=element)
              self.__get_unordered_list_items(data=element)
              self.__get_toc(data=element)
              self.__get_slide(data=element)
              self.__get_box_like(env='box', data=element)
              self.__get_box_like(env='note', data=element)
              self.__get_box_like(env='table', data=element)
              self.__get_box_like(env='figure', data=element)
              self.__get_box_like(env='video', data=element)
        self.custom = True
      except YAMLError:
        print('No valid definition of theme has been found')
    self.__get_css()

  def get_slide_decorators_metadata(self, decorator, name):
    """Get the slide decorators (headers, footers, sidebars) metadata placeholders.

    Parameters
    ----------
    decorator: str
      header|footer|sidebar
    name: str
      header-1, header-2, ... footer-1,...

    Returns
    -------
    str:
      metadata placeholder
    """
    metadata = getattr(self, 'slide_' + decorator + '_metadata')[name]
    placeholders = []
    for data in metadata:
      placeholders.append(r'$' + data + r'[' + metadata[data] + ']')
    return ''.join(placeholders)

  def get_slide_transition(self):
    """Get slide transition (positioning) informations.

    Returns
    -------
    dict:
      dictionary of the slides-transition informations
    """
    infos = {}
    transition = self.__get_attribute(attribute='transition', css_list=self.slide)
    if transition == '':
      transition = 'horizontal'
    infos['transition'] = transition

    width = self.__get_attribute(attribute='width', css_list=self.slide)
    if width == '':
      width = '900px'
    infos['width'] = int(width.strip('px'))

    height = self.__get_attribute(attribute='height', css_list=self.slide)
    if height == '':
      height = '900px'
    infos['height'] = int(height.strip('px'))

    offset = self.__get_attribute(attribute='data-offset', css_list=self.slide)
    if offset == '':
      offset = '10'
    infos['offset'] = int(offset)

    scale = self.__get_attribute(attribute='scale', css_list=self.slide)
    if scale == '':
      scale = '1'
    infos['scale'] = int(scale)

    infos['data-x'] = self.__get_attribute(attribute='data-x', css_list=self.slide)
    infos['data-y'] = self.__get_attribute(attribute='data-y', css_list=self.slide)
    infos['data-z'] = self.__get_attribute(attribute='data-z', css_list=self.slide)
    infos['data-rotate-x'] = self.__get_attribute(attribute='data-rotate-x', css_list=self.slide)
    infos['data-rotate-y'] = self.__get_attribute(attribute='data-rotate-y', css_list=self.slide)
    infos['data-rotate-z'] = self.__get_attribute(attribute='data-rotate-z', css_list=self.slide)
    return infos



#!/usr/bin/env python
"""
presentation.py, module definition of Presentation class.
"""
from __future__ import print_function
from collections import OrderedDict
import os
# from shutil import copyfile, copytree, rmtree
from shutil import copytree, rmtree
from yaml import load_all, YAMLError
from yattag import Doc, indent
from chapter import Chapter
from metadata import Metadata
from parser import Parser
from position import Position
from section import Section
from slide import Slide
from subsection import Subsection
from theme import Theme


class Presentation(object):
  """
  Presentation object.

  Attributes
  ----------
  chapters_number: int
  """
  chapters_number = 0

  @classmethod
  def reset(cls):
    """Reset to default state."""
    cls.chapters_number = 0
    Theme.reset()
    Chapter.reset()

  def __init__(self):
    """
    Attributes
    ----------
    metadata: dict
      presentation metadata; each element of the dictionary if a dict with ['value', 'user'] items: value contains
      the metadata value and user indicates if the value comes from user (if True) or from defaults (if False).
    """
    self.reset()
    self.metadata = {'title': Metadata(name='title', value=''),
                     'subtitle': Metadata(name='subtitle', value=''),
                     'authors': Metadata(name='authors', value=[]),
                     'authors_short': Metadata(name='authors_short', value=[]),
                     'emails': Metadata(name='emails', value=[]),
                     'affiliations': Metadata(name='affiliations', value=[]),
                     'affiliations_short': Metadata(name='affiliations_short', value=[]),
                     'logo': Metadata(name='logo', value=''),
                     'timer': Metadata(name='timer', value=''),
                     'location': Metadata(name='location', value=''),
                     'location_short': Metadata(name='location_short', value=''),
                     'date': Metadata(name='date', value=''),
                     'conference': Metadata(name='conference', value=''),
                     'conference_short': Metadata(name='conference_short', value=''),
                     'session': Metadata(name='session', value=''),
                     'session_short': Metadata(name='session_short', value=''),
                     'max_time': Metadata(name='max_time', value='25'),
                     'total_slides_number': Metadata(name='total_slides_number', value=''),
                     'dirs_to_copy': Metadata(name='dirs_to_copy', value=[]),
                     'toc': Metadata(name='toc', value=OrderedDict()),
                     'toc_depth': Metadata(name='toc_depth', value='2'),
                     'chaptertitle': Metadata(name='chaptertitle', value=''),
                     'chapternumber': Metadata(name='chapternumber', value=''),
                     'sectiontitle': Metadata(name='sectiontitle', value=''),
                     'sectionnumber': Metadata(name='sectionnumber', value=''),
                     'subsectiontitle': Metadata(name='subsectiontitle', value=''),
                     'subsectionnumber': Metadata(name='subsectionnumber', value=''),
                     'slidetitle': Metadata(name='slidetitle', value=''),
                     'slidenumber': Metadata(name='slidenumber', value=''),
                     'custom': Metadata(name='custom-[0-9]*', value='')}
    self.theme = Theme()
    self.parser = Parser()
    self.chapters = []
    self.position = Position()
    return

  def __str__(self):
    strings = ['Chapters number ' + str(Presentation.chapters_number)]
    strings.append('Sections number ' + str(Chapter.sections_number))
    strings.append('Subsections number ' + str(Section.subsections_number))
    strings.append('Slides number ' + str(Subsection.slides_number))
    for chapter in self.chapters:
      strings.append(str(chapter))
    return '\n'.join(strings)

  def __update_toc(self):
    """Update TOC after a new chapter (the last one) has been added."""
    self.metadata['toc'].value[self.chapters[-1].title] = self.chapters[-1].toc

  def __get_metadata(self, source):
    """
    Get metadata from source stream.

    Parameters
    ----------
    source: str
    """
    codeblocks = self.parser.tokenizer(source=source, re_search=self.parser.regexs['codeblock'])
    yamlblocks = self.parser.tokenizer(source=source, re_search=self.parser.regexs['yamlblock'], exclude=codeblocks)
    yaml_source = ''.join([block['match'].group().strip('---') for block in yamlblocks])
    try:
      for data in load_all(yaml_source):
        if 'metadata' in data:
          for element in data['metadata']:
            for key in element:
              if key in self.metadata:
                self.metadata[key].update_value(value=element[key])
    except YAMLError:
      print('No valid definition of metadata has been found')

  def __get_theme(self, source):
    """
    Get theme from source stream.

    Parameters
    ----------
    source: str
    """
    codeblocks = self.parser.tokenizer(source=source, re_search=self.parser.regexs['codeblock'])
    yamlblocks = self.parser.tokenizer(source=source, re_search=self.parser.regexs['yamlblock'], exclude=codeblocks)
    self.theme.get(''.join([block['match'].group().strip('---') for block in yamlblocks]))

  def __add_chapter(self, chapter):
    """
    Add a chapter to the pesentation.

    Parameters
    ----------
    chapter: Chapter
    """
    Presentation.chapters_number += 1
    self.chapters.append(chapter)
    self.__update_toc()
    return

  def __check_bad_sectioning(self, tokens):
    """Check if the presentation has a bad sectioning.

    Parameters
    ----------
    tokens: Parser.tokens
    source: str
    """
    if '$titlepage' not in tokens['slides'][0]['match'].group().lower():
      if tokens['slides'][0]['start'] < tokens['subsections'][0]['start'] or tokens['slides'][0]['start'] < tokens['sections'][0]['start'] or tokens['slides'][0]['start'] < tokens['chapters'][0]['start']:
        print('Warning: found bad presentation sectioning!')
        print('The slide definition:')
        print(tokens['slides'][0]['match'].group() + "\n")
        print('is placed before the first defined chapter/section/subsection.')
        print('All contents before the first defined chapter/section/subsection is omitted!')
        print()
    return

  def __put_html_tag_head(self, doc, tag, text, config):
    """Put head tag into html doc.

    Parameters
    ----------
    doc: Doc()
    tag: Tag()
    config : MatisseConfig
      MaTiSSe configuration
    """
    with tag('head'):
      doc.stag('meta', charset='utf-8')
      doc.stag('meta', author=' and '.join(self.metadata['authors'].value))
      with tag('title'):
        text(self.metadata['title'].value)
      doc.stag('meta', subtitle=self.metadata['subtitle'].value)
      doc.stag('link', rel='stylesheet', href='css/normalize.css')
      if config.highlight:
        doc.stag('link', rel='stylesheet', href='js/highlight/styles/' + config.highlight_style)
      doc.stag('link', rel='stylesheet', href='css/theme.css')

  def __put_html_tags_scripts(self, doc, tag, config):
    """Put final tags for scripts into html doc.

    Parameters
    ----------
    doc: Doc()
    tag: Tag()
    config : MatisseConfig
      MaTiSSe configuration
    """
    with tag('script'):
      doc.attr(src='js/countDown.js')
    with tag('script'):
      doc.attr(src='js/impress.js')
    with tag('script'):
      doc.asis('impress().init();')
    if config.online_mathjax:
      with tag('script'):
        doc.attr(('type', 'text/javascript'))
        doc.attr(src='http://cdn.mathjax.org/mathjax/latest/MathJax.js?config=TeX-AMS-MML_HTMLorMML')
    else:
      with tag('script'):
        doc.attr(('type', 'text/x-mathjax-config'))
        doc.text("""
          MathJax.Hub.Config({
            extensions: ["tex2jax.js"],
            jax: ["input/TeX", "output/HTML-CSS"],
            tex2jax: {
              inlineMath: [ ['$','$'] ],
              displayMath: [ ['$$','$$'] ],
              processEscapes: true
            },
            "HTML-CSS": { availableFonts: ["Neo-Euler"] }
          });
        """)
      with tag('script'):
        doc.attr(('type', 'text/javascript'))
        doc.attr(src='js/MathJax/MathJax.js')
    if config.highlight:
      with tag('script'):
        doc.attr(src='js/highlight/highlight.pack.js')
      with tag('script'):
        doc.text("""hljs.initHighlightingOnLoad();""")

  def __put_html_slide_decorators(self, tag, doc, decorator, position=None, overtheme=None, current=None):
    """Put html data of headers, footers and sidebars.

    Parameters
    ----------
    doc: Doc
    tag: tag
    decorator: {header, footer, sidebar}
    position: {'L','R'}
      sidebars position, L => left, R => right
    current: list
    """
    if overtheme is not None and overtheme.custom:
      theme = overtheme
    else:
      theme = self.theme
    # decorators = getattr(self.theme, 'slide_' + decorator)
    decorators = getattr(theme, 'slide_' + decorator)
    for decor in sorted(decorators):
      insert = True
      # position check for sidebars
      if decorator == 'sidebar' and position is not None:
        for css in decorators[decor]:
          for key in css:
            if 'position' in key.lower():
              pos = css[key]
              break
        insert = pos.lower() == position.lower()
      # active check
      for css in decorators[decor]:
        for key in css:
          if 'active' in key.lower():
            insert = insert and css[key].lower() == 'yes'
      if insert:
        # placeholders = self.theme.get_slide_decorators_metadata(decorator=decorator, name=decor)
        placeholders = theme.get_slide_decorators_metadata(decorator=decorator, name=decor)
        for metadata in self.metadata:
          placeholders = self.metadata[metadata].parse(parser=self.parser, source=placeholders, toc_depth=self.metadata['toc_depth'].value, max_time=self.metadata['max_time'].value, current=current)
        with tag('div', klass='slide-' + decor):
          doc.asis(placeholders)

  def parse(self, config, source):
    """Parse presentation from source stream.

    Parameters
    ----------
    config : MatisseConfig
      MaTiSSe configuration
    source: str
    """
    complete_source = self.parser.includes(source=source)
    self.__get_metadata(source=complete_source)
    self.__get_theme(source=complete_source)
    new_theme = Theme()
    new_theme.set_from(other=self.theme)
    tokens = self.parser.tokenize(source=complete_source)
    self.__check_bad_sectioning(tokens=tokens)
    chapters_number = 0
    sections_number = 0
    subsections_number = 0
    slides_number = 0
    titlepage_inserted = False
    for chap in tokens['chapters']:
      chapters_number += 1
      slide_local_numbers = [0, 0, 0]
      if chap['match'].group('expr'):
        chapter = Chapter(number=chapters_number, title=chap['match'].group('expr'))
      else:
        chapter = Chapter(number=chapters_number, title='')
      for sec in tokens['sections']:
        if sec['start'] >= chap['start'] and sec['start'] <= chap['end_next']:
          sections_number += 1
          slide_local_numbers[1] = 0
          slide_local_numbers[2] = 0
          section = Section(number=sections_number, title=sec['match'].group('expr'))
          for subsec in tokens['subsections']:
            if subsec['start'] >= sec['start'] and subsec['start'] <= sec['end_next']:
              subsections_number += 1
              slide_local_numbers[2] = 0
              subsection = Subsection(number=subsections_number, title=subsec['match'].group('expr'))
              for sld in tokens['slides']:
                if '$titlepage' in sld['match'].group().lower() and not titlepage_inserted:
                  slide = Slide(number=0,
                                title='titlepage',
                                contents=complete_source[sld['end']:sld['end_next']])
                  slide.get_overtheme(parser=self.parser)
                  if slide.overtheme.copy_from_theme is not None and slide.overtheme.copy_from_theme:
                    slide.overtheme.copy_from(other=self.theme)
                  self.position.update_position(presentation_theme=self.theme, overtheme=slide.overtheme)
                  slide.set_position(position=self.position.position)
                  subsection.add_slide(slide=slide)
                  titlepage_inserted = True
                else:
                  if sld['start'] >= subsec['start'] and sld['start'] <= subsec['end_next']:
                    slide_local_numbers[0] += 1
                    slide_local_numbers[1] += 1
                    slide_local_numbers[2] += 1
                    if slide_local_numbers[0] == 1 and config.toc_at_chap_beginning is not None:
                      slides_number += 1
                      self.position.update_position(presentation_theme=self.theme)
                      subsection.add_slide(slide=Slide(number=slides_number,
                                                       position=self.position.position,
                                                       title='Table of Contents',
                                                       contents='$toc[depth:' + str(config.toc_at_chap_beginning) + ']'))
                    if slide_local_numbers[1] == 1 and config.toc_at_sec_beginning is not None:
                      slides_number += 1
                      self.position.update_position(presentation_theme=self.theme)
                      subsection.add_slide(slide=Slide(number=slides_number,
                                                       position=self.position.position,
                                                       title='Table of Contents',
                                                       contents='$toc[depth:' + str(config.toc_at_sec_beginning) + ']'))
                    if slide_local_numbers[2] == 1 and config.toc_at_subsec_beginning is not None:
                      slides_number += 1
                      self.position.update_position(presentation_theme=self.theme)
                      subsection.add_slide(slide=Slide(number=slides_number,
                                                       position=self.position.position,
                                                       title='Table of Contents',
                                                       contents='$toc[depth:' + str(config.toc_at_subsec_beginning) + ']'))
                    slides_number += 1
                    slide = Slide(number=slides_number,
                                  title=sld['match'].group('expr'),
                                  contents=complete_source[sld['end']:sld['end_next']])
                    slide.get_overtheme(parser=self.parser)
                    if slide.overtheme.copy_from_theme is not None and slide.overtheme.copy_from_theme:
                      slide.overtheme.copy_from(other=self.theme)
                    self.position.update_position(presentation_theme=self.theme, overtheme=slide.overtheme)
                    slide.set_position(position=self.position.position)
                    subsection.add_slide(slide=slide)
              section.add_subsection(subsection=subsection)
          chapter.add_section(section=section)
      self.__add_chapter(chapter=chapter)
      self.metadata['total_slides_number'].update_value(value=str(Subsection.slides_number))

  def to_html(self, config):
    """Generate a html stream of the whole presentation.

    Parameters
    ----------
    config : MatisseConfig
      MaTiSSe configuration
    """
    doc, tag, text = Doc().tagtext()
    doc.asis('<!DOCTYPE html>')
    with tag('html'):
      doc.attr(title=self.metadata['title'].value)
      self.__put_html_tag_head(doc=doc, tag=tag, text=text, config=config)
      with tag('body', onload="resetCountdown(" + str(self.metadata['max_time'].value) + ");"):
        with tag('div', id='impress'):
          # numbering: [local_chap, local_sec, local_subsec, local_slide]
          current = [0, 0, 0, 0]
          for chapter in self.chapters:
            current[0] += 1
            current[1] = 0
            current[2] = 0
            current[3] = 0
            self.metadata['chaptertitle'].update_value(value=chapter.title)
            self.metadata['chapternumber'].update_value(value=chapter.number)
            for section in chapter.sections:
              current[1] += 1
              current[2] = 0
              current[3] = 0
              self.metadata['sectiontitle'].update_value(value=section.title)
              self.metadata['sectionnumber'].update_value(value=section.number)
              for subsection in section.subsections:
                current[2] += 1
                current[3] = 0
                self.metadata['subsectiontitle'].update_value(value=subsection.title)
                self.metadata['subsectionnumber'].update_value(value=subsection.number)
                for slide in subsection.slides:
                  current[3] += 1
                  self.metadata['slidetitle'].update_value(value=slide.title)
                  self.metadata['slidenumber'].update_value(value=slide.number)
                  with doc.tag('div'):
                    chapter.put_html_attributes(doc=doc)
                    section.put_html_attributes(doc=doc)
                    subsection.put_html_attributes(doc=doc)
                    slide.put_html_attributes(doc=doc)
                    self.__put_html_slide_decorators(tag=tag, doc=doc, decorator='header', current=current, overtheme=slide.overtheme)
                    self.__put_html_slide_decorators(tag=tag, doc=doc, decorator='sidebar', position='L', current=current, overtheme=slide.overtheme)
                    slide.to_html(doc=doc, parser=self.parser, metadata=self.metadata, current=current)
                    self.__put_html_slide_decorators(tag=tag, doc=doc, decorator='sidebar', position='R', current=current, overtheme=slide.overtheme)
                    self.__put_html_slide_decorators(tag=tag, doc=doc, decorator='footer', current=current, overtheme=slide.overtheme)
        self.__put_html_tags_scripts(doc=doc, tag=tag, config=config)
    # source = re.sub(r"<li>(?P<item>.*)</li>", r"<li><span>\g<item></span></li>", source)
    html = indent(doc.getvalue())
    return html

  def save(self, config, output):
    """Save the html form of presentation into external file.

    Parameters
    ----------
    config : MatisseConfig
      MaTiSSe configuration
    output : str
      output path
    """
    if not os.path.exists(output):
      os.makedirs(output)
    with open(os.path.join(output, 'index.html'), 'w') as html:
      html.write(self.to_html(config=config))
    # copy user defined directories if set
    if len(self.metadata['dirs_to_copy'].value) > 0:
      for data in self.metadata['dirs_to_copy'].value:
        if os.path.exists(os.path.join(output, data)):
          rmtree(os.path.join(output, data))
        copytree(data, os.path.join(output, data))
    with open(os.path.join(output, 'css/theme.css'), 'w') as css_theme:
      css_theme.writelines(self.theme.css)
      for chapter in self.chapters:
        for section in chapter.sections:
          for subsection in section.subsections:
            for slide in subsection.slides:
              if slide.overtheme.custom:
                css_theme.writelines(slide.overtheme.css)
    return



#!/usr/bin/env python
"""
slide.py, module definition of Slide class.
"""
from __future__ import print_function
from box import Box
from columns import Columns
from figure import Figure
from markdown_utils import markdown2html
from note import Note
from table import Table
from theme import Theme
from video import Video


class Slide(object):
  """
  Slide object.
  """

  @classmethod
  def reset(cls):
    """Reset to default state."""
    return

  def __init__(self, number, position=None, title=None, contents=None):
    """"
    Paramters
    ---------
    number: int
      slide global numeration
    position: dict
      position dictionary containing {'x': posx, 'y': posy, 'z': posz, 'rotx': rotx, 'roty': roty, 'rotz': rotz, 'scale': scaling}
    title: str
    contents: str
    """
    self.number = number
    self.position = None
    self.set_position(position)
    self.title = title
    self.contents = contents
    self.overtheme = Theme()
    return

  def __str__(self):
    strings = [str(self.title)]
    strings.append(str(self.contents))
    return ''.join(strings)

  def get_overtheme(self, parser):
    """Get eventaul overtheme definition.

    Parameters
    ----------
    parser: Parser
    """
    codeblocks = parser.tokenizer(source=self.contents, re_search=parser.regexs['codeblock'])
    yamlblocks = parser.tokenizer(source=self.contents, re_search=parser.regexs['yamlblock'], exclude=codeblocks)
    if len(yamlblocks) > 0:
      self.overtheme.get(source=''.join([block['match'].group().strip('---') for block in yamlblocks]),
                         name='overtheme',
                         div_id='slide-' + str(self.number),
                         default=False)
      purged_contents = self.contents[:yamlblocks[0]['start']]
      for b, yamlblock in enumerate(yamlblocks[:-1]):
        purged_contents += self.contents[yamlblock['end']:yamlblocks[b + 1]['start']]
      purged_contents += self.contents[yamlblocks[-1]['end']:]
      self.contents = purged_contents

  def set_position(self, position):
    """Set slide position.

    Parameters
    ----------
    position: dict
      position dictionary containing {'x': posx, 'y': posy, 'z': posz, 'rotx': rotx, 'roty': roty, 'rotz': rotz, 'scale': scaling}
    """
    if position is not None:
      self.position = {}
      for key in position:
        self.position[key] = position[key]

  def put_html_attributes(self, doc):
    """Put html attibutes of the slide.

    Parameters
    ----------
    doc: Doc
    """
    doc.attr(('id', 'slide-' + str(self.number)))
    doc.attr(('title', str(self.title)))
    doc.attr(('class', 'step slide'))
    doc.attr(('data-x', str(self.position['x'])))
    doc.attr(('data-y', str(self.position['y'])))
    doc.attr(('data-z', str(self.position['z'])))
    doc.attr(('data-scale', str(self.position['scale'])))
    doc.attr(('data-rotate-x', str(self.position['rotx'])))
    doc.attr(('data-rotate-y', str(self.position['roty'])))
    doc.attr(('data-rotate-z', str(self.position['rotz'])))
    return

  def to_html(self, doc, parser, metadata, current):
    """Generate html from self.

    Parameters
    ----------
    doc: Doc
    parser: Parser
    metatadata: dict
      presentation metadata
    current: list
    """
    def _parse_env(Env, re_search, source):
      codeblocks = parser.tokenizer(source=source, re_search=parser.regexs['codeblock'])
      codes = parser.tokenizer(source=source, re_search=parser.regexs['code'], exclude=codeblocks)
      yamlblocks = parser.tokenizer(source=source, re_search=parser.regexs['yamlblock'], exclude=codeblocks + codes)
      envs = parser.tokenizer(source=source, re_search=re_search, exclude=codeblocks + yamlblocks + codes)
      if len(envs) > 0:
        parsed_source = source[:envs[0]['start']]
        for e, env in enumerate(envs[:-1]):
          current = Env(source=env['match'].group())
          parsed_source += current.to_html() + source[env['end']:envs[e + 1]['start']]
        current = Env(source=envs[-1]['match'].group())
        parsed_source += current.to_html() + source[envs[-1]['end']:]
        return parsed_source
      return source

    html = self.contents
    for meta in metadata:
      html = metadata[meta].parse(parser=parser, source=html, toc_depth=metadata['toc_depth'].value, max_time=metadata['max_time'].value, current=current)
    html = _parse_env(Env=Box, re_search=Box.regexs['box'], source=html)
    html = _parse_env(Env=Note, re_search=Note.regexs['note'], source=html)
    html = _parse_env(Env=Figure, re_search=Figure.regexs['figure'], source=html)
    html = _parse_env(Env=Table, re_search=Table.regexs['table'], source=html)
    html = _parse_env(Env=Video, re_search=Video.regexs['video'], source=html)
    html = _parse_env(Env=Columns, re_search=Columns.regexs['columns'], source=html)
    with doc.tag('div', klass='slide-content'):
      doc.asis(markdown2html(source=html))
    return
